@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -202,35 +202,12 @@ static size_t
   max_profile_size = 0,
   virtual_anonymous_memory = 0;
 
-#if defined _MSC_VER
-static void *MSCMalloc(size_t size)
-{
-  return(malloc(size));
-}
-
-static void *MSCRealloc(void* ptr, size_t size)
-{
-  return(realloc(ptr,size));
-}
-
-static void MSCFree(void* ptr)
-{
-  free(ptr);
-}
-#endif
-
 static MagickMemoryMethods
   memory_methods =
   {
-#if defined _MSC_VER
-    (AcquireMemoryHandler) MSCMalloc,
-    (ResizeMemoryHandler) MSCRealloc,
-    (DestroyMemoryHandler) MSCFree,
-#else
     (AcquireMemoryHandler) malloc,
     (ResizeMemoryHandler) realloc,
     (DestroyMemoryHandler) free,
-#endif
     (AcquireAlignedMemoryHandler) NULL,
     (RelinquishAlignedMemoryHandler) NULL
   };
