@@ -30,17 +30,17 @@ extern "C" {
 #define MagickLibVersion  0x712
 #define MagickLibVersionText  "7.1.2"
 #define MagickLibVersionNumber  10,0,2
-#define MagickLibAddendum  "-14 (Beta)"
+#define MagickLibAddendum  "-15"
 #define MagickLibInterface  10
 #define MagickLibMinInterface  10
 #define MagickPlatform  "x86_64"
 #define MagickppLibVersionText  "7.1.2"
 #define MagickppLibVersionNumber  5:0:0
-#define MagickppLibAddendum  "-14 (Beta)"
+#define MagickppLibAddendum  "-15"
 #define MagickppLibInterface  5
 #define MagickppLibMinInterface  5
-#define MagickGitRevision  "23562"
-#define MagickReleaseDate  "2026-01-31"
+#define MagickGitRevision  "23686"
+#define MagickReleaseDate  "2026-02-22"
 #define MagickAuthoritativeLicense  \
   "https://imagemagick.org/license/"
 #define MagickAuthoritativeURL  "https://imagemagick.org"
