@@ -29,7 +29,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -220,13 +220,11 @@ MagickPrivate XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
     subnode[MagickPathExtent],
     tag[MagickPathExtent];
 
-  ssize_t
-    i;
-
   size_t
     number_components;
 
   ssize_t
+    i,
     j;
 
   XMLTreeInfo
@@ -423,9 +421,7 @@ static void DestroyXMLTreeRoot(XMLTreeInfo *xml_info)
     **attributes;
 
   ssize_t
-    i;
-
-  ssize_t
+    i,
     j;
 
   XMLTreeRoot
@@ -533,9 +529,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
     offset;
 
   size_t
-    i;
-
-  size_t
+    i,
     length;
 
   ssize_t
@@ -589,7 +583,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
           break;
       }
       if (LocaleCompare(filename,"-") != 0)
-        file=close(file);
+        file=close_utf8(file);
       if (xml == (char *) NULL)
         return((char *) NULL);
       if (file == -1)
@@ -607,7 +601,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
     xml=(char *) AcquireQuantumMemory(length+MagickPathExtent,sizeof(*xml));
   if (xml == (char *) NULL)
     {
-      file=close(file);
+      file=close_utf8(file);
       return((char *) NULL);
     }
   map=MapBlob(file,ReadMode,0,length);
@@ -622,7 +616,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
       for (i=0; i < length; i+=(size_t) count)
       {
         count=read(file,xml+i,(size_t) MagickMin(length-i,(size_t)
-          MAGICK_SSIZE_MAX));
+          MagickMaxBufferExtent));
         if (count <= 0)
           {
             count=0;
@@ -632,14 +626,14 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
       }
       if (i < length)
         {
-          file=close(file)-1;
+          file=close_utf8(file)-1;
           xml=(char *) RelinquishMagickMemory(xml);
           return((char *) NULL);
         }
     }
   xml[length]='\0';
   if (LocaleCompare(filename,"-") != 0)
-    file=close(file);
+    file=close_utf8(file);
   if (file == -1)
     xml=(char *) RelinquishMagickMemory(xml);
   return(xml);
@@ -706,9 +700,7 @@ MagickExport const char *GetXMLTreeAttribute(XMLTreeInfo *xml_info,
   const char *tag)
 {
   ssize_t
-    i;
-
-  ssize_t
+    i,
     j;
 
   XMLTreeRoot
@@ -934,13 +926,11 @@ MagickPrivate XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,
     subnode[MagickPathExtent],
     tag[MagickPathExtent];
 
-  ssize_t
-    i;
-
   size_t
     number_components;
 
   ssize_t
+    i,
     j;
 
   XMLTreeInfo
@@ -1224,13 +1214,11 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     c,
     encoding;
 
-  ssize_t
-    i;
-
   size_t
     extent;
 
   ssize_t
+    i,
     j;
 
   utf8=(char *) AcquireQuantumMemory(*length+1,sizeof(*utf8));
@@ -1268,7 +1256,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
       }
     if (c < 0x80)
       {
-        utf8[j]=c;
+        utf8[j]=(char) c;
         j++;
         continue;
       }
@@ -1279,7 +1267,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     for (bits=0; byte != 0; byte/=2)
       bits++;
     bits=(bits-2)/5;
-    utf8[j++]=(0xFF << (7-bits)) | (c >> (6*bits));
+    utf8[j++]=(char) ((0xFF << (7-bits)) | (c >> (6*bits)));
     while (bits != 0)
     {
       bits--;
@@ -1297,24 +1285,20 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
 static char *ParseEntities(char *xml,char **entities,int state)
 {
   char
-    *entity;
+    *entity,
+    *p,
+    *q;
 
   int
     byte,
     c;
 
-  char
-    *p,
-    *q;
-
-  ssize_t
-    i;
-
   size_t
     extent,
     length;
 
   ssize_t
+    i,
     offset;
 
   /*
@@ -1362,7 +1346,7 @@ static char *ParseEntities(char *xml,char **entities,int state)
             continue;
           }
         if (c < 0x80)
-          *(xml++)=c;
+          *(xml++)=(char) c;
         else
           {
             /*
@@ -1547,9 +1531,7 @@ static void ParseProcessingInstructions(XMLTreeRoot *root,char *xml,
     *target;
 
   ssize_t
-    i;
-
-  ssize_t
+    i,
     j;
 
   target=xml;
@@ -1628,9 +1610,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
     *v;
 
   ssize_t
-    i;
-
-  ssize_t
+    i,
     j;
 
   n=(char *) NULL;
@@ -1884,6 +1864,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
   char
     **attribute,
     **attributes,
+    *p,
     *tag,
     *utf8;
 
@@ -1894,17 +1875,12 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  char
-    *p;
-
-  ssize_t
-    i;
-
   size_t
     ignore_depth,
     length;
 
   ssize_t
+    i,
     j,
     l;
 
@@ -1962,7 +1938,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
             utf8=DestroyString(utf8);
             return(&root->root);
           }
-        p+=strcspn(p,XMLWhitespace "/>");
+        p+=(ptrdiff_t) strcspn(p,XMLWhitespace "/>");
         while (isspace((int) ((unsigned char) *p)) != 0)
           *p++='\0';
         if (((isalpha((int) ((unsigned char) *p)) != 0) || (*p == '_')) &&
@@ -2000,13 +1976,13 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
               attributes[l+2]=(char *) NULL;
               attributes[l+1]=(char *) NULL;
               attributes[l]=p;
-              p+=strcspn(p,XMLWhitespace "=/>");
+              p+=(ptrdiff_t) strcspn(p,XMLWhitespace "=/>");
               if ((*p != '=') && (isspace((int) ((unsigned char) *p)) == 0))
                 attributes[l]=ConstantString("");
               else
                 {
                   *p++='\0';
-                  p+=strspn(p,XMLWhitespace "=");
+                  p+=(ptrdiff_t) strspn(p,XMLWhitespace "=");
                   c=(*p);
                   if ((c == '"') || (c == '\''))
                     {
@@ -2088,7 +2064,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                     ignore_depth++;
                     (void) DestroyXMLTreeAttributes(attributes);
                   }
-                *p=c;
+                *p=(char) c;
               }
             else
               {
@@ -2108,7 +2084,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
             Close tag.
           */
           tag=p+1;
-          p+=strcspn(tag,XMLWhitespace ">")+1;
+          p+=(ptrdiff_t) strcspn(tag,XMLWhitespace ">")+1;
           c=(*p);
           if ((c == '\0') && (terminal != '>'))
             {
@@ -2126,9 +2102,9 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
             }
           if (ignore_depth > 0)
             ignore_depth--;
-          *p=c;
+          *p=(char) c;
           if (isspace((int) ((unsigned char) *p)) != 0)
-            p+=strspn(p,XMLWhitespace);
+            p+=(ptrdiff_t) strspn(p,XMLWhitespace);
         }
       else
         if (strncmp(p,"!--",3) == 0)
@@ -2155,7 +2131,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
               p=strstr(p,"]]>");
               if (p != (char *) NULL)
                 {
-                  p+=2;
+                  p+=(ptrdiff_t) 2;
                   if (ignore_depth == 0)
                     ParseCharacterContent(root,tag+8,(size_t) (p-tag-10),'c');
                 }
@@ -2177,7 +2153,7 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                      ((l != 0) && ((*p != ']') ||
                      (*(p+strspn(p+1,XMLWhitespace)+1) != '>'))));
                   l=(ssize_t) ((*p == '[') ? 1 : l))
-                p+=strcspn(p+1,"[]>")+1;
+                p+=(ptrdiff_t) strcspn(p+1,"[]>")+1;
                 if ((*p == '\0') && (terminal != '>'))
                   {
                     (void) ThrowMagickException(exception,GetMagickModule(),
@@ -2297,18 +2273,14 @@ MagickExport XMLTreeInfo *NewXMLTreeTag(const char *tag)
   XMLTreeRoot
     *root;
 
-  root=(XMLTreeRoot *) AcquireMagickMemory(sizeof(*root));
-  if (root == (XMLTreeRoot *) NULL)
-    return((XMLTreeInfo *) NULL);
+  root=(XMLTreeRoot *) AcquireCriticalMemory(sizeof(*root));
   (void) memset(root,0,sizeof(*root));
   root->root.tag=(char *) NULL;
   if (tag != (char *) NULL)
     root->root.tag=ConstantString(tag);
   root->node=(&root->root);
   root->root.content=ConstantString("");
-  root->entities=(char **) AcquireMagickMemory(sizeof(predefined_entities));
-  if (root->entities == (char **) NULL)
-    return((XMLTreeInfo *) NULL);
+  root->entities=(char **) AcquireCriticalMemory(sizeof(predefined_entities));
   (void) memcpy(root->entities,predefined_entities,sizeof(predefined_entities));
   root->root.attributes=sentinel;
   root->attributes=(char ***) root->root.attributes;
@@ -2419,9 +2391,7 @@ MagickPrivate XMLTreeInfo *SetXMLTreeAttribute(XMLTreeInfo *xml_info,
   const char *tag,const char *value)
 {
   ssize_t
-    i;
-
-  ssize_t
+    i,
     j;
 
   assert(xml_info != (XMLTreeInfo *) NULL);
@@ -2587,13 +2557,11 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
   const char
     *attribute;
 
-  ssize_t
-    i;
-
   size_t
     offset;
 
   ssize_t
+    i,
     j;
 
   content=(char *) "";
@@ -2687,21 +2655,17 @@ static char *XMLTreeTagToXML(XMLTreeInfo *xml_info,char **source,size_t *length,
 
 MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
 {
-  char
-    *xml;
-
   char
     *p,
-    *q;
-
-  ssize_t
-    i;
+    *q,
+    *xml;
 
   size_t
     extent,
     length;
 
   ssize_t
+    i,
     j,
     k;
 
