@@ -23,7 +23,7 @@
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -126,6 +126,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
     length;
 
   ssize_t
+    columns,
     count,
     y;
 
@@ -203,6 +204,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
   scene=0;
   status=MagickTrue;
   stream=NULL;
+  columns=(ssize_t) MagickMin(image->columns,canvas_image->columns);
   do
   {
     /*
@@ -263,7 +265,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
                 image->columns,1,exception);
               if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                 break;
-              for (x=0; x < (ssize_t) image->columns; x++)
+              for (x=0; x < columns; x++)
               {
                 SetPixelRed(image,GetPixelRed(canvas_image,p),q);
                 SetPixelGreen(image,GetPixelGreen(canvas_image,p),q);
@@ -343,7 +345,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
                   image->columns,1,exception);
                 if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                   break;
-                for (x=0; x < (ssize_t) image->columns; x++)
+                for (x=0; x < columns; x++)
                 {
                   switch (quantum_type)
                   {
@@ -423,7 +425,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
                 image->columns,1,exception);
               if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                 break;
-              for (x=0; x < (ssize_t) image->columns; x++)
+              for (x=0; x < columns; x++)
               {
                 SetPixelGray(image,GetPixelGray(canvas_image,p),q);
                 p+=(ptrdiff_t) GetPixelChannels(canvas_image);
@@ -478,7 +480,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
                     image->columns,1,exception);
                   if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                     break;
-                  for (x=0; x < (ssize_t) image->columns; x++)
+                  for (x=0; x < columns; x++)
                   {
                     SetPixelAlpha(image,GetPixelAlpha(canvas_image,p),q);
                     p+=(ptrdiff_t) GetPixelChannels(canvas_image);
@@ -568,7 +570,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
                 image->columns,1,exception);
               if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                 break;
-              for (x=0; x < (ssize_t) image->columns; x++)
+              for (x=0; x < columns; x++)
               {
                 SetPixelRed(image,GetPixelRed(canvas_image,p),q);
                 p+=(ptrdiff_t) GetPixelChannels(canvas_image);
@@ -642,7 +644,7 @@ static Image *ReadGRAYImage(const ImageInfo *image_info,
                     image->columns,1,exception);
                   if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
                     break;
-                  for (x=0; x < (ssize_t) image->columns; x++)
+                  for (x=0; x < columns; x++)
                   {
                     SetPixelAlpha(image,GetPixelAlpha(canvas_image,p),q);
                     p+=(ptrdiff_t) GetPixelChannels(canvas_image);
@@ -1000,7 +1002,10 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
         status=OpenBlob(image_info,image,scene == 0 ? WriteBinaryBlobMode :
           AppendBinaryBlobMode,exception);
         if (status == MagickFalse)
-          return(status);
+          {
+            quantum_info=DestroyQuantumInfo(quantum_info);
+            return(status);
+          }
         for (y=0; y < (ssize_t) image->rows; y++)
         {
           const Quantum
@@ -1030,7 +1035,10 @@ static MagickBooleanType WriteGRAYImage(const ImageInfo *image_info,
             status=OpenBlob(image_info,image,scene == 0 ? WriteBinaryBlobMode :
               AppendBinaryBlobMode,exception);
             if (status == MagickFalse)
-              return(status);
+              {
+                quantum_info=DestroyQuantumInfo(quantum_info);
+                return(status);
+              }
             for (y=0; y < (ssize_t) image->rows; y++)
             {
               const Quantum
