@@ -17,13 +17,13 @@
 %                                 July 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -48,10 +48,11 @@
 #include "MagickCore/image.h"
 #include "MagickCore/image-private.h"
 #include "MagickCore/list.h"
+#include "MagickCore/locale-private.h"
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/module.h"
-#include "MagickCore/profile.h"
+#include "MagickCore/profile-private.h"
 #include "MagickCore/splay-tree.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
@@ -177,10 +178,10 @@ static int stringnicmp(const char *p,const char *q,size_t n)
       break;
     i=(*p);
     if (islower((int) ((unsigned char) i)) != 0)
-      i=LocaleUppercase(i);
+      i=LocaleToUppercase((int) i);
     j=(*q);
     if (islower((int) ((unsigned char) j)) != 0)
-      j=LocaleUppercase(j);
+      j=LocaleToUppercase((int) j);
     if (i != j)
       break;
     n--;
@@ -189,7 +190,7 @@ static int stringnicmp(const char *p,const char *q,size_t n)
     p++;
     q++;
   }
-  return(LocaleUppercase((int) *p)-LocaleUppercase((int) *q));
+  return(LocaleToUppercase((int) *p)-LocaleToUppercase((int) *q));
 }
 
 static size_t convertHTMLcodes(char *s)
@@ -212,7 +213,7 @@ static size_t convertHTMLcodes(char *s)
       }
   if ((length == 0) || (s == (char *) NULL) || (*s == '\0'))
     return(0);
-  if ((length > 3) && (s[1] == '#') && (sscanf(s,"&#%d;",&value) == 1))
+  if ((length > 3) && (s[1] == '#') && (MagickSscanf(s,"&#%d;",&value) == 1))
     {
       size_t
         o;
@@ -226,7 +227,7 @@ static size_t convertHTMLcodes(char *s)
       }
       if (o < 6)
         (void) memmove(s+1,s+1+o,strlen(s+1+o)+1);
-      *s=value;
+      *s=(char) value;
       return(o);
     }
   for (i=0; i < (ssize_t) (sizeof(html_codes)/sizeof(html_codes[0])); i++)
@@ -272,11 +273,9 @@ static char *super_fgets(char **b, size_t *blen, Image *file)
         tlen=(size_t) (q-p);
         len<<=1;
         buffer=(unsigned char *) ResizeQuantumMemory(p,len+2UL,sizeof(*p));
+        p=(unsigned char *) NULL;
         if (buffer == (unsigned char *) NULL)
-          {
-            p=(unsigned char *) RelinquishMagickMemory(p);
-            break;
-          }
+          break;
         p=buffer;
         q=p+tlen;
       }
@@ -315,11 +314,8 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
     state,
     next;
 
-  unsigned char
-    dataset;
-
-  unsigned int
-    recnum;
+  MagickBooleanType
+    status;
 
   MagickOffsetType
     savedpos,
@@ -335,6 +331,12 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
   TokenInfo
     *token_info;
 
+  unsigned char
+    dataset;
+
+  unsigned int
+    recnum;
+
   dataset = 0;
   recnum = 0;
   line = (char *) AcquireQuantumMemory(inputlen,sizeof(*line));
@@ -342,6 +344,7 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
     return(-1);
   newstr = name = token = (char *) NULL;
   savedpos = 0;
+  status=MagickTrue;
   token_info=AcquireTokenInfo();
   while (super_fgets(&line,&inputlen,ifile)!=NULL)
   {
@@ -380,11 +383,13 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
                 recnum = (unsigned int) StringToUnsignedLong(newstr);
                 break;
               case 2:
-                name=(char *) AcquireQuantumMemory(strlen(newstr)+MagickPathExtent,
-                  sizeof(*name));
-                if (name)
-                  (void) strcpy(name,newstr);
+              {
+                size_t extent = strlen(newstr)+MagickPathExtent;
+                name=(char *) AcquireQuantumMemory(extent,sizeof(*name));
+                if (name != (char *) NULL)
+                  (void) CopyMagickString(name,newstr,extent);
                 break;
+              }
             }
             s++;
           }
@@ -415,7 +420,7 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
                   if ((ssize_t) codes_length > len)
                     len=0;
                   else
-                    len-=codes_length;
+                    len-=(ssize_t) codes_length;
                 }
             }
 
@@ -436,21 +441,21 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
                     currentpos = TellBlob(ofile);
                     if (currentpos < 0)
                       {
-                        line=DestroyString(line);
-                        return(-1);
+                        status=MagickFalse;
+                        break;
                       }
                     offset=SeekBlob(ofile,savedpos,SEEK_SET);
                     if (offset < 0)
                       {
-                        line=DestroyString(line);
-                        return(-1);
+                        status=MagickFalse;
+                        break;
                       }
                     (void) WriteBlobMSBLong(ofile,(unsigned int) diff);
                     offset=SeekBlob(ofile,currentpos,SEEK_SET);
                     if (offset < 0)
                       {
-                        line=DestroyString(line);
-                        return(-1);
+                        status=MagickFalse;
+                        break;
                       }
                     savedolen = 0L;
                   }
@@ -495,7 +500,10 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
                     /* patch in a fake length for now and fix it later */
                     savedpos = TellBlob(ofile);
                     if (savedpos < 0)
-                      return(-1);
+                      {
+                        status=MagickFalse;
+                        break;
+                      }
                     (void) WriteBlobMSBLong(ofile,0xFFFFFFFFU);
                     outputlen += 4;
                     savedolen = outputlen;
@@ -525,6 +533,8 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
       newstr=DestroyString(newstr);
     if (name != (char *) NULL)
       name=DestroyString(name);
+    if (status == MagickFalse)
+      break;
   }
   token_info=DestroyTokenInfo(token_info);
   if (token != (char *) NULL)
@@ -553,7 +563,7 @@ static ssize_t parse8BIM(Image *ifile, Image *ofile)
         return(-1);
       savedolen = 0L;
     }
-  return outputlen;
+  return(status == MagickFalse ? -1 : outputlen);
 }
 
 static char *super_fgets_w(char **b, size_t *blen, Image *file)
@@ -588,6 +598,7 @@ static char *super_fgets_w(char **b, size_t *blen, Image *file)
         tlen=(size_t) (q-p);
         len<<=1;
         buffer=(unsigned char *) ResizeQuantumMemory(p,len+2,sizeof(*p));
+        p=(unsigned char *) NULL;
         if (buffer == (unsigned char *) NULL)
           break;
         p=buffer;
@@ -597,7 +608,7 @@ static char *super_fgets_w(char **b, size_t *blen, Image *file)
   }
   *b=(char *) p;
   *blen=0;
-  if ((*b) != (char *) NULL)
+  if (p != (unsigned char *) NULL)
     {
       size_t
         tlen;
@@ -638,6 +649,9 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
     savedolen = 0L,
     outputlen = 0L;
 
+  MagickBooleanType
+    status;
+
   MagickOffsetType
     savedpos,
     currentpos;
@@ -653,6 +667,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
   newstr = name = token = (char *) NULL;
   savedpos = 0;
   token_info=AcquireTokenInfo();
+  status=MagickTrue;
   while (super_fgets_w(&line,&inputlen,ifile) != NULL)
   {
     state=0;
@@ -725,7 +740,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
                   if ((ssize_t) codes_length > len)
                     len=0;
                   else
-                    len-=codes_length;
+                    len-=(ssize_t) codes_length;
                 }
             }
 
@@ -742,17 +757,28 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
                     MagickOffsetType
                       offset;
 
-                    ssize_t diff = outputlen - savedolen;
+                    ssize_t
+                      diff = outputlen - savedolen;
+
                     currentpos = TellBlob(ofile);
                     if (currentpos < 0)
-                      return(-1);
+                      {
+                        status=MagickFalse;
+                        break;
+                      }
                     offset=SeekBlob(ofile,savedpos,SEEK_SET);
                     if (offset < 0)
-                      return(-1);
+                      {
+                        status=MagickFalse;
+                        break;
+                      }
                     (void) WriteBlobMSBLong(ofile,(unsigned int) diff);
                     offset=SeekBlob(ofile,currentpos,SEEK_SET);
                     if (offset < 0)
-                      return(-1);
+                      {
+                        status=MagickFalse;
+                        break;
+                      }
                     savedolen = 0L;
                   }
                 if (outputlen & 1)
@@ -782,7 +808,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
 
                     n=0;
                     outputlen += len;
-                    while (len--)
+                    while (len-- > 0)
                       (void) WriteBlobByte(ofile,(unsigned char) token[n++]);
 
                     if (outputlen & 1)
@@ -796,7 +822,10 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
                     /* patch in a fake length for now and fix it later */
                     savedpos = TellBlob(ofile);
                     if (savedpos < 0)
-                      return(-1);
+                      {
+                        status=MagickFalse;
+                        break;
+                      }
                     (void) WriteBlobMSBLong(ofile,0xFFFFFFFFU);
                     outputlen += 4;
                     savedolen = outputlen;
@@ -813,7 +842,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
                     outputlen += 5;
                     n=0;
                     outputlen += len;
-                    while (len--)
+                    while (len-- > 0)
                       (void) WriteBlobByte(ofile,(unsigned char) token[n++]);
                   }
               }
@@ -826,6 +855,8 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
       newstr=DestroyString(newstr);
     if (name != (char *) NULL)
       name=DestroyString(name);
+    if (status == MagickFalse)
+      break;
   }
   token_info=DestroyTokenInfo(token_info);
   if (token != (char *) NULL)
@@ -854,7 +885,7 @@ static ssize_t parse8BIMW(Image *ifile, Image *ofile)
         return(-1);
       savedolen = 0L;
     }
-  return(outputlen);
+  return(status == MagickFalse ? -1 : outputlen);
 }
 
 /* some defines for the different JPEG block types */
@@ -1187,11 +1218,11 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1251,17 +1282,9 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       else
         CopyBlob(image,buff);
-      profile=BlobToStringInfo(GetBlobStreamData(buff),(size_t)
-        GetBlobSize(buff));
-      if (profile == (StringInfo *) NULL)
-        {
-          blob=(unsigned char *) DetachBlob(buff->blob);
-          blob=(unsigned char *) RelinquishMagickMemory(blob);
-          buff=DestroyImage(buff);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        }
-      status=SetImageProfile(image,"8bim",profile,exception);
-      profile=DestroyStringInfo(profile);
+      profile=BlobToProfileStringInfo("8bim",GetBlobStreamData(buff),(size_t)
+        GetBlobSize(buff),exception);
+      SetImageProfilePrivate(image,profile,exception);
       blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
@@ -1322,22 +1345,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       else
         CopyBlob(image,buff);
-      profile=BlobToStringInfo(GetBlobStreamData(buff),(size_t)
-        GetBlobSize(buff));
-      if (profile == (StringInfo *) NULL)
-        {
-          blob=(unsigned char *) DetachBlob(buff->blob);
-          blob=(unsigned char *) RelinquishMagickMemory(blob);
-          buff=DestroyImage(buff);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        }
-      status=SetImageProfile(image,name,profile,exception);
-      profile=DestroyStringInfo(profile);
+      profile=BlobToProfileStringInfo(name,GetBlobStreamData(buff),(size_t)
+        GetBlobSize(buff),exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
       blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
-      if (status == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     }
   if ((LocaleCompare(image_info->magick,"ICC") == 0) ||
       (LocaleCompare(image_info->magick,"ICM") == 0))
@@ -1353,17 +1366,9 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       AttachBlob(buff->blob,blob,length);
       CopyBlob(image,buff);
-      profile=BlobToStringInfo(GetBlobStreamData(buff),(size_t)
-        GetBlobSize(buff));
-      if (profile == (StringInfo *) NULL)
-        {
-          blob=(unsigned char *) DetachBlob(buff->blob);
-          blob=(unsigned char *) RelinquishMagickMemory(blob);
-          buff=DestroyImage(buff);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        }
-      (void) SetImageProfile(image,"icc",profile,exception);
-      profile=DestroyStringInfo(profile);
+      profile=BlobToProfileStringInfo("icc",GetBlobStreamData(buff),(size_t)
+        GetBlobSize(buff),exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
       blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
@@ -1381,17 +1386,9 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       AttachBlob(buff->blob,blob,length);
       CopyBlob(image,buff);
-      profile=BlobToStringInfo(GetBlobStreamData(buff),(size_t)
-        GetBlobSize(buff));
-      if (profile == (StringInfo *) NULL)
-        {
-          blob=(unsigned char *) DetachBlob(buff->blob);
-          blob=(unsigned char *) RelinquishMagickMemory(blob);
-          buff=DestroyImage(buff);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        }
-      (void) SetImageProfile(image,"iptc",profile,exception);
-      profile=DestroyStringInfo(profile);
+      profile=BlobToProfileStringInfo("iptc",GetBlobStreamData(buff),(size_t)
+        GetBlobSize(buff),exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
       blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
@@ -1409,22 +1406,17 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       AttachBlob(buff->blob,blob,length);
       CopyBlob(image,buff);
-      profile=BlobToStringInfo(GetBlobStreamData(buff),(size_t)
-        GetBlobSize(buff));
-      if (profile == (StringInfo *) NULL)
-        {
-          blob=(unsigned char *) DetachBlob(buff->blob);
-          blob=(unsigned char *) RelinquishMagickMemory(blob);
-          buff=DestroyImage(buff);
-          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-        }
-      (void) SetImageProfile(image,"xmp",profile,exception);
-      profile=DestroyStringInfo(profile);
+      profile=BlobToProfileStringInfo("xmp",GetBlobStreamData(buff),(size_t)
+        GetBlobSize(buff),exception);
+      (void) SetImageProfilePrivate(image,profile,exception);
       blob=(unsigned char *) DetachBlob(buff->blob);
       blob=(unsigned char *) RelinquishMagickMemory(blob);
       buff=DestroyImage(buff);
     }
-  (void) CloseBlob(image);
+  if (CloseBlob(image) == MagickFalse)
+    status=MagickFalse;
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -1615,22 +1607,20 @@ static size_t GetIPTCStream(unsigned char **info,size_t length)
   int
     c;
 
-  ssize_t
-    i;
+  size_t
+    extent,
+    info_length,
+    tag_length;
 
   unsigned char
     *p;
 
-  size_t
-    extent,
-    info_length;
+  ssize_t
+    i;
 
   unsigned int
     marker;
 
-  size_t
-    tag_length;
-
   p=(*info);
   extent=length;
   if ((*p == 0x1c) && (*(p+1) == 0x02))
@@ -1642,23 +1632,23 @@ static size_t GetIPTCStream(unsigned char **info,size_t length)
   {
     if (strncmp((const char *) p,"8BIM",4))
       break;
-    p+=4;
+    p+=(ptrdiff_t) 4;
     extent-=4;
     marker=(unsigned int) (*p) << 8 | *(p+1);
-    p+=2;
+    p+=(ptrdiff_t) 2;
     extent-=2;
     c=*p++;
     extent--;
     c|=0x01;
     if ((size_t) c >= extent)
       break;
-    p+=c;
-    extent-=c;
+    p+=(ptrdiff_t) c;
+    extent=(size_t) ((ssize_t) extent-c);
     if (extent < 4)
       break;
     tag_length=(((size_t) *p) << 24) | (((size_t) *(p+1)) << 16) |
       (((size_t) *(p+2)) << 8) | ((size_t) *(p+3));
-    p+=4;
+    p+=(ptrdiff_t) 4;
     extent-=4;
     if (tag_length > extent)
       break;
@@ -1669,7 +1659,7 @@ static size_t GetIPTCStream(unsigned char **info,size_t length)
       }
     if ((tag_length & 0x01) != 0)
       tag_length++;
-    p+=tag_length;
+    p+=(ptrdiff_t) tag_length;
     extent-=tag_length;
   }
   /*
@@ -1756,17 +1746,17 @@ iptc_find:
         /*
           Short format.
         */
-        tag_length=((long) c) << 8;
+        tag_length=(size_t) (c << 8);
         c=(*p++);
         length--;
         if (length == 0)
           break;
         info_length++;
-        tag_length|=(long) c;
+        tag_length|=(unsigned int) c;
       }
     if (tag_length > (length+1))
       break;
-    p+=tag_length;
+    p+=(ptrdiff_t) tag_length;
     length-=tag_length;
     if (length == 0)
       break;
@@ -1882,14 +1872,13 @@ static const tag_spec tags[] = {
   { 219, "Custom Field 20" }
 };
 
-static int formatIPTC(Image *ifile, Image *ofile)
+static void formatIPTC(Image *ifile, Image *ofile)
 {
   char
     temp[MagickPathExtent];
 
   unsigned int
-    foundiptc,
-    tagsfound;
+    foundiptc;
 
   unsigned char
     recnum,
@@ -1911,9 +1900,8 @@ static int formatIPTC(Image *ifile, Image *ofile)
     c;
 
   foundiptc = 0; /* found the IPTC-Header */
-  tagsfound = 0; /* number of tags found */
 
-  c = ReadBlobByte(ifile);
+  c=ReadBlobByte(ifile);
   while (c != EOF)
   {
     if (c == 0x1c)
@@ -1921,22 +1909,22 @@ static int formatIPTC(Image *ifile, Image *ofile)
     else
       {
         if (foundiptc)
-          return(-1);
+          return;
         else
           {
-            c=0;
+            c=ReadBlobByte(ifile);
             continue;
           }
       }
 
     /* we found the 0x1c tag and now grab the dataset and record number tags */
-    c = ReadBlobByte(ifile);
+    c=ReadBlobByte(ifile);
     if (c == EOF)
-      return(-1);
+      return;
     dataset = (unsigned char) c;
-    c = ReadBlobByte(ifile);
+    c=ReadBlobByte(ifile);
     if (c == EOF)
-      return(-1);
+      return;
     recnum = (unsigned char) c;
     /* try to match this record to one of the ones in our named table */
     for (i=0; i< tagcount; i++)
@@ -1953,9 +1941,9 @@ static int formatIPTC(Image *ifile, Image *ofile)
     */
     c=ReadBlobByte(ifile);
     if (c == EOF)
-      return(-1);
+      return;
     if (c & (unsigned char) 0x80)
-      return(0);
+      return;
     else
       {
         int
@@ -1963,23 +1951,23 @@ static int formatIPTC(Image *ifile, Image *ofile)
 
         c0=ReadBlobByte(ifile);
         if (c0 == EOF)
-          return(-1);
+          return;
         taglen = (c << 8) | c0;
       }
     if (taglen < 0)
-      return(-1);
+      return;
     /* make a buffer to hold the tag datand snag it from the input stream */
     str=(unsigned char *) AcquireQuantumMemory((size_t) (taglen+
       MagickPathExtent),sizeof(*str));
     if (str == (unsigned char *) NULL)
-      return(0);
+      return;
     for (tagindx=0; tagindx<taglen; tagindx++)
     {
       c=ReadBlobByte(ifile);
       if (c == EOF)
         {
           str=(unsigned char *) RelinquishMagickMemory(str);
-          return(-1);
+          return;
         }
       str[tagindx] = (unsigned char) c;
     }
@@ -1995,12 +1983,8 @@ static int formatIPTC(Image *ifile, Image *ofile)
     (void) WriteBlobString(ofile,temp);
     formatString( ofile, (char *)str, taglen );
     str=(unsigned char *) RelinquishMagickMemory(str);
-
-    tagsfound++;
-
     c=ReadBlobByte(ifile);
   }
-  return((int) tagsfound);
 }
 
 static int readWordFromBuffer(char **s, ssize_t *len)
@@ -2301,7 +2285,7 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (LocaleCompare(image_info->magick,"8BIM") == 0)
     {
@@ -2334,6 +2318,8 @@ static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
       assert(exception != (ExceptionInfo *) NULL);
       assert(exception->signature == MagickCoreSignature);
       status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
+      if (status == MagickFalse)
+        return(status);
       info=GetStringInfoDatum(profile);
       length=GetStringInfoLength(profile);
       length=GetIPTCStream(&info,length);
