@@ -17,13 +17,13 @@
 %                                 March 2012                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization         %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://imagemagick.org/script/license.php                               %
+%    https://imagemagick.org/license/                                         %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -184,11 +184,11 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   (void) ResetImagePage(image,"0x0+0+0");
   if ((image->columns != 0) && (image->rows != 0))
@@ -373,7 +373,7 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
           error->message,"`%s'",image_info->filename);
       pango_layout_set_markup(layout,caption,-1);
     }
-  if (draw_info->interline_spacing > 0)
+  if (draw_info->interline_spacing != 0)
     pango_layout_set_spacing(layout,ScalePangoValue(
       draw_info->interline_spacing,image->resolution.x));
   pango_layout_context_changed(layout);
@@ -384,22 +384,24 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
   if (image->columns == 0)
     {
       pango_layout_get_extents(layout,NULL,&extent);
-      image->columns=(extent.x+extent.width+PANGO_SCALE/2)/PANGO_SCALE+2*page.x;
+      image->columns=(size_t) ((extent.x+extent.width+PANGO_SCALE/2)/
+        PANGO_SCALE+2*page.x);
     }
   else
     {
-      image->columns-=2*page.x;
+      image->columns=(size_t) ((ssize_t) image->columns-2*page.x);
       pango_layout_set_width(layout,ScalePangoValue((double) image->columns,
         image->resolution.x));
     }
   if (image->rows == 0)
     { 
       pango_layout_get_extents(layout,NULL,&extent);
-      image->rows=(extent.y+extent.height+PANGO_SCALE/2)/PANGO_SCALE+2*page.y;
+      image->rows=(size_t) ((extent.y+extent.height+PANGO_SCALE/2)/
+        PANGO_SCALE+2*page.y);
     }
   else
     {
-      image->rows-=2*page.y;
+      image->rows=(size_t) ((ssize_t) image->rows-2*page.y);
       pango_layout_set_height(layout,ScalePangoValue((double) image->rows,
         image->resolution.y));
     }
@@ -425,7 +427,7 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
   cairo_set_operator(cairo_image,CAIRO_OPERATOR_CLEAR);
   cairo_paint(cairo_image);
   cairo_set_operator(cairo_image,CAIRO_OPERATOR_OVER);
-  cairo_translate(cairo_image,page.x,page.y);
+  cairo_translate(cairo_image,(double) page.x,(double) page.y);
   cairo_set_source_rgba(cairo_image,QuantumScale*draw_info->fill.red,
     QuantumScale*draw_info->fill.green,QuantumScale*draw_info->fill.blue,
     QuantumScale*draw_info->fill.alpha);
@@ -464,13 +466,13 @@ static Image *ReadPANGOImage(const ImageInfo *image_info,
         Disassociate alpha.
       */
       gamma=QuantumScale*fill_color.alpha;
-      gamma=PerceptibleReciprocal(gamma);
+      gamma=MagickSafeReciprocal(gamma);
       fill_color.blue*=gamma;
       fill_color.green*=gamma;
       fill_color.red*=gamma;
       CompositePixelOver(image,&fill_color,fill_color.alpha,q,(double)
         GetPixelAlpha(image,q),q);
-      q+=GetPixelChannels(image);
+      q+=(ptrdiff_t) GetPixelChannels(image);
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
       break;
